/* Data for newsgroups */

/* $Id: group.h,v 2.0 2001/07/26 16:35:12 tony Exp $ */

#ifndef GROUP_H
#define GROUP_H

#include <stdio.h>

#ifndef DEFNS_H
#include "defns.h"
#endif
#ifndef HASHTREE_H
#include "hashtree.h"
#endif

typedef enum {
    group_Virginal,
    group_Allocated,		/* Group has been allocated to a thread, but
				   thread hasn't started yet */
    group_Entering,		/* A thread is now active on this group, but
				   article numbers are not yet valid */
    group_Downloading,
    group_AllDone
} group_Status;

typedef enum {
    artnum_Unknown,
    artnum_Unavailable,
    artnum_Available,
    artnum_Duplicate,
    artnum_Touched
} artnum_Status;

typedef struct {
    artnum_Status status;
    MsgIdNode *msgid;
} group_artnum;
    

struct GroupData {
    char *name;
    long int first_available;
    long int last_available;
    long int last_downloaded;	/* 0 means download all */
    long int last_requested;
    long int interested_threads;
    group_Status status;
    group_artnum *artnums;
};

GroupData *group_new(const char *name, long int last_downloaded);

void group_delete(GroupData *);

/* Returns whether any articles need to be downloaded; identifier is server id
 * for logging */
BOOL group_got_stats(GroupData *, long int first_available, long int
	last_available, const char *identifier, int thread_ix);

/* Indicate that a message is available for download */
void group_add_message_id(GroupData *, long int artnum, const char *msgid);

/* Get next article number available for download, 0 for all done, -1 if any
 * still have status artnum_Unknown */
long int group_get_next_article(GroupData *, char const **msgid);

/* Once all articles in a group have been XHDR'd we know that any left marked
 * Unknown are now Unavailable */
void group_prune_unavailable(GroupData *);

/* Update data when a message has been downloaded */
void group_have_downloaded(GroupData *, long int artnum);

/* Writes newsrc data into file when fetch is complete */
void group_write_newsrc(GroupData *, FILE *fp);

#endif /* ! GROUP_H */
