/* $Id: intercom.c,v 2.0 2001/07/26 16:35:12 tony Exp $ */

#include "intercom.h"

Intercom *intercom_new(rw_buf *buf)
{
    Intercom *ic = XMALLOC(sizeof(Intercom));

    ic->buf = buf;
    ic->data.data = ic->str_data = NULL;
    return ic;
}

void intercom_delete(Intercom *ic)
{
    if (!ic)
	return;
    free(ic->str_data);
    free(ic);
}

void intercom_send_command(rw_buf *buf, intercom_Command code, int thread_idx,
	const char *data, ...)
{
    va_list ap;
    
    va_start(ap, data);
    intercom_send_vcommand(buf, code, thread_idx, data, ap);
    va_end(ap);
}

void intercom_send_vcommand(rw_buf *buf, intercom_Command code, int thread_idx,
	const char *data, va_list ap)
{
    char *longstr;
    char *formstr;
    char headstr[8];

    if (vasprintf(&formstr, data, ap) < 0)
    {
	fprintf(stderr, 
		_("Can't send interprocess command (out of memory?)\n"));
	exit(1);
    }
    snprintf(headstr, 8, N_("%d %c "), thread_idx, code);
    longstr = XSTRDUPCAT(headstr, formstr);
    if (rw_write_line(buf, longstr) <= 0)
    {
	fprintf(stderr, _("Can't send interprocess command: %s\n %s\n"),
		strerror(errno), longstr);
	/*
	perror(_("Can't send interprocess command"));
	*/
	exit(1);
    }
    free(formstr);
    free(longstr);
}

const intercom_data *intercom_get_command(Intercom *ic)
{
    const char *line;
    size_t len;
    
    free(ic->str_data);
    ic->str_data = NULL;
    line = rw_read_line(ic->buf);
    if (!line)
    {
	deblog_error(_("Can't read interprocess command: %s\n"),
		strerror(errno));
	return NULL;
    }
    ic->data.command = command_Null;
    if (sscanf(line, N_("%d %c"), &ic->data.thread_idx, (char *)
		&ic->data.command) < 2
	    || ic->data.command < command_First
	    || ic->data.command > command_Last)
    {
	deblog_error(_("Received bad IPC command line: %s\n"), line);
	free(ic->str_data);
	ic->str_data = NULL;
	return NULL;
    }
    ic->str_data = XSTRDUP(strchr(strchr(line, ' ') + 1, ' ') + 1);
    /* Log message may have been sent with \n replaced by \r, so switch back */
    len = strlen(ic->str_data) - 1;
    if (ic->str_data[len] == '\r')
	ic->str_data[len] = '\n';
    ic->data.data = ic->str_data;
    /*
    if (ic->data.command != command_Log)
    {
	deblog_debug(_("Received IPC line: %s\n"), line);
    }
    */
    return &ic->data;
}

