/* For exchange of commands between processes via pipes */

/* $Id: intercom.h,v 2.0 2001/07/26 16:35:12 tony Exp $ */

#ifndef INTERCOM_H
#define INTERCOM_H

#ifndef COMMON_H
#include "common.h"
#endif

/* Each command consists of space separated fields:
 *
 * Thread idx (can be anything for parent, because children don't listen to
 * anything else)
 * 
 * Command code
 * 
 * Data depending on command
 */

typedef enum {
    command_Null = 0,
    command_First = 'A',
    /* From child to parent */
    command_Ready = command_First,	/* No data, logged on ready to receive
					   first command */
    command_Resolved,			/* IP address (dotted-quad) requested
					   by command_Resolve */
    command_Connected,
    command_Authenticated,
    command_GroupArticleStats,		/* Group idx, first, last art numbers */
    command_GroupContains,		/* Group idx, article no, Message-ID */
    command_GroupComplete,		/* Group idx:- all IDs read */
    command_GetNextArticle,		/* Request for next Message-ID */
    command_HaveDownloaded,		/* Group idx, article no, filename */
    command_Uploaded,
    command_Duplicate,
    command_BadUpload,
    command_NoUploads,			/* If posting fails because of the
					   message, it's child's responsibility
					   to move it to "failed" directory. If
					   posting fails in general, it sends
					   this message instead, and parent
					   should skip uploading rest of
					   server's outgoing articles. */
    command_Log,			/* Level, message */
    command_AmountRcvd,			/* Bytes received from server since arg
					   2; timeval */
    command_AmountSent,			/* As above, but bytes sent */
    command_Error,			/* Critical error with message to be
					   logged; implies command_AllDone */
    command_QuietError,			/* As error, without a message */
    command_AllDone,			/* Usually in response to
					   command_Shutdown */

    /* From parent to child */
    command_Resolve,			/* Address (name) to be resolved */
    command_Connect,			/* Port; Server (given in dotted-quad);
					   entry to add to Path header or !;
					   whether to pipelien (0/1)
					 */
    command_EnterGroup,			/* Group ix, name, whether to return
					   article stats to parent (0/1) */
    command_Auth,			/* User name, password */
    command_LogOn = command_Auth,
    command_ModeReader,			/* Send MODE READER command */
    command_GetArticles,		/* Send series of GroupContains;
					   group ix, first, last */
    command_GetArticle,			/* Group ix, article no, message-id */
    command_Upload,			/* Full filename */
    command_Throttle,			/* Caught up with list of IDs, or using
					   too much bandwidth, pause for given
					   time (ms) */
    command_Shutdown,			/* Pull any messages remaining in
					   queue, then log off server and stop
					 */

    command_Last = command_Shutdown
} intercom_Command;

struct intercom_data {
    intercom_Command command;
    int thread_idx;
    const char *data;
};

struct Intercom {
    rw_buf *buf;
    intercom_data data;
    char *str_data;		/* Needed because data (string) in
				   intercom_data is const */
};

Intercom *intercom_new(rw_buf *);

void intercom_delete(Intercom *);

void intercom_send_command(rw_buf *, intercom_Command, int thread_idx, const
	char *data, ...);

void intercom_send_vcommand(rw_buf *, intercom_Command, int thread_idx, const
	char *data, va_list);

const intercom_data *intercom_get_command(Intercom *);

#endif /* INTERCOM_H */
