/* $Id: kiddy.c,v 2.0 2001/07/26 16:35:12 tony Exp $ */

#include "common.h"

#include <signal.h>
#include <sys/wait.h>

#include "child.h"
#include "intercom.h"
#include "kiddy.h"

static KiddyData *kiddy_table;

/* Parent reads commands from all children from this */
static Intercom *kiddy_intercom = NULL;

static BOOL kiddy_we_are_child = FALSE;

KiddyData *kiddy_get_available(void)
{
    int i;

    for (i = 0; i < maincf_get_max_threads(); ++i)
    {
	if (!kiddy_table[i].thread)
	{
	    return &kiddy_table[i];
	}
    }
    return NULL;
}

void kiddy_detach_thread(KiddyData *kiddy)
{
    kiddy->thread = NULL;
}

#if 0
static void kiddy_kill_all(void)
{
    int i;

/*    if (kiddy_we_are_child) */
	return;
    deblog_info(_("Killing all child processes\n"));
    signal(SIGCHLD, SIG_IGN);
    for (i = 0; i < maincf_get_max_threads(); ++i)
    {
	if (kiddy_table[i].pid != -1)
	{
	    kill(kiddy_table[i].pid, SIGTERM);
	    kiddy_table[i].pid = -1;
	}
    }
}
#endif /* Comment block */

static void kiddy_sigchld_handler(int signum)
{
    int active = 0;
    int i;
    
    UNUSED(signum);

    deblog_critical(_("A child process has died unexpectedly\n"));
    for (i = 0; i < maincf_get_max_threads(); ++i)
    {
	if (kiddy_table[i].pid != -1)
	{
	    if (waitpid(kiddy_table[i].pid, NULL, WNOHANG))
	    {
		/* nsthread_delete(kiddy_table[i].thread); */
		kiddy_table[i].pid = -1;
	    }
	    else
		++active;
	}
    }
    if (!active)
    {
	deblog_error(_("No child processes left, exiting\n"));
    }
}

static pid_t kiddy_spawn(int thread_idx, int read_from, int write_to, int
	*pipe_table)
{
    pid_t pid;
    int i;
    
    pid = fork();
    switch (pid)
    {
	case 0:
	    /* Close all pipes we're not using */
	    for (i = 0; i < maincf_get_max_threads(); ++i)
	    {
		if (pipe_table[i] != -1)
		    close(pipe_table[i]);
	    }
	    kiddy_we_are_child = TRUE;
	    child_start(thread_idx, read_from, write_to);
	    break;
	case -1:
	   deblog_error(_("Unable to fork: %s\n"), strerror(errno));
	    break;
	default:
	    /* Don't need to read from parent's copy of read_from, close it */
	    close(read_from);
	    return pid;
    }
    return 0;
}

void kiddy_spawn_all()
{
    int i;
    int write_to[2];
    /* Need a table of all parent > child pipes so that children can close ones
     * they aren't using */
    int *pipe_table;
    
    deblog_info(_("Starting %d child processes\n"), maincf_get_max_threads());
    kiddy_table = XMALLOC(maincf_get_max_threads() * sizeof(KiddyData));
    pipe_table = XMALLOC(maincf_get_max_threads() * sizeof(int));
    for (i = 0; i < maincf_get_max_threads(); ++i)
    {
	kiddy_table[i].thread = NULL;
	kiddy_table[i].buf = NULL;
	kiddy_table[i].pid = -1;
	kiddy_table[i].thread_idx = i;
	pipe_table[i] = -1;
    }
    /* atexit(kiddy_kill_all); */
    signal(SIGCHLD, kiddy_sigchld_handler);
    XPIPE(write_to);
    for (i = 0; i < maincf_get_max_threads(); ++i)
    {
	int pipefd[2];

	XPIPE(pipefd);
	deblog_debug(_("Child %d will read from %d and be written to via %d\n"),
		i, pipefd[0], pipefd[1]);
	pipe_table[i] = pipefd[1];
	kiddy_table[i].pid = kiddy_spawn(i, pipefd[0], write_to[1], pipe_table);
    }
    for (i = 0; i < maincf_get_max_threads(); ++i)
    {
	kiddy_table[i].buf = rw_create_buf(pipe_table[i], FALSE, FALSE, -1);
    }
    /* We don't need to write to write_to */
    close(write_to[1]);
    kiddy_intercom = intercom_new(rw_create_buf(write_to[0], FALSE, TRUE, -1));
}

const intercom_data *kiddy_get_command(void)
{
    return intercom_get_command(kiddy_intercom);
}

