/* $Id: maincf.c,v 2.2 2001/10/25 18:16:15 tony Exp $ */

#include "common.h"
#include "parserc.h"

#ifndef MAXTHREADS
#define MAXTHREADS 3
#endif

static struct {
    long int max_threads;
    long int threads_per_server;
    BOOL pipeline;
    char *path;
    char *history_method;
    char *post_method;
} maincf_data;

/*
max_threads			 Absolute maximum number of threads to run
threads_per_server		 Maximum number of threads for any one server
*/

void maincf_init(void)
{
    maincf_data.max_threads = maincf_data.threads_per_server = MAXTHREADS;
    maincf_data.path = maincf_data.history_method = NULL;
    maincf_data.pipeline = TRUE;
    deblog_info(_("Loading main config file\n"));
    
    parserc_register_int(N_("max_threads"), &maincf_data.max_threads);
    parserc_register_int(N_("threads_per_server"),
	    &maincf_data.threads_per_server);
    parserc_register_bool(N_("pipeline"), &maincf_data.pipeline);
    parserc_register_string(N_("path"), &maincf_data.path);
    parserc_register_string(N_("history_method"), &maincf_data.history_method);
    parserc_register_string(N_("post_method"), &maincf_data.post_method);
    
    parserc_read_named(MAIN_CONF, deblog_Level_Warning);

    if (maincf_data.max_threads < 1)
    {
	deblog_warning(_("Too few max threads (%d), changing to 1\n"),
		maincf_data.max_threads);
	maincf_data.max_threads = 1;
    }
    if (maincf_data.threads_per_server < 1)
    {
	deblog_warning(_("Too few threads per server (%d), changing to 1\n"),
		maincf_data.threads_per_server);
	maincf_data.threads_per_server = 1;
    }
    if (maincf_data.threads_per_server > maincf_data.max_threads)
    {
	deblog_warning(_("Threads per server (%d) greater than max threads (%d)"
		    ", truncating\n"), maincf_data.threads_per_server,
		    maincf_data.max_threads);
	maincf_data.threads_per_server = maincf_data.max_threads;
    }

    deblog_debug(_("max_threads = %d\n"), maincf_data.max_threads);
    deblog_debug(_("threads_per_server = %d\n"),
	    maincf_data.threads_per_server);
}

long int maincf_get_threads_per_server(void)
{
    return maincf_data.threads_per_server;
}

long int maincf_get_max_threads(void)
{
    return maincf_data.max_threads;
}

BOOL maincf_get_pipeline(void)
{
    return maincf_data.pipeline;
}

const char *maincf_get_path(void)
{
    return maincf_data.path;
}

const char *maincf_get_history_method(void)
{
    return maincf_data.history_method;
}

const char *maincf_get_post_method(void)
{
    return maincf_data.post_method;
}

void maincf_reset_max_threads(long int mt)
{
    maincf_data.max_threads = mt;
}
