/* Interface for outputting via curses instead of stdout/stderr */

/* $Id: nscurses.h,v 2.3 2001/08/31 16:38:54 tony Exp $ */

#ifndef NSCURSES_H 
#define NSCURSES_H

#ifndef COMMON_H
#include <common.h>
#endif

#ifdef HAVE_CURSES_H

/* Whether we're using curses interface */
BOOL nscurses_active(void);

/* Start using curses, set up windows etc; needs to know how many threads are
 * in use to divide up screen */
void nscurses_start(int nthreads);

/* Stop using curses and clear screen etc */
void nscurses_stop(void);

/* As above, but a bit more abrupt */
void nscurses_emergency_stop(void);
    
/* Output a message for deblog (thread_idx == -1 for parent) */
void nscurses_deblog(int thread_idx, deblog_level_t level, const char *format,
	va_list ap);

/* Output stats (rates are xntokilomeg strings) */
void nscurses_stats(long int downloaded, long int total_downloads,
	const char *download_rate,
	long int uploaded, long int total_uploads, const char *upload_rate);

/* Indicate which server a thread is connected to */
void nscurses_server(int thread_idx, const char *name);

#else /* ! HAVE_CURSES */

#define nscurses_active FALSE
#define nscurses_start(n)
#define nscurses_stop
#define nscurses_emergency_stop
#define nscurses_deblog(t, l, f, a)
#define nscurses_stats(d, td, dr, u, tu, ur)
#define nscurses_server(t, n)

#endif /* HAVE_CURSES_H */

#endif /* NSCURSES_H */
