/* $Id: nsthread.c,v 2.2 2001/08/13 21:28:29 tony Exp $ */

#include "common.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "group.h"
#include "intercom.h"
#include "kiddy.h"
#include "nscurses.h"
#include "nsthread.h"
#include "server.h"

static NSThread **nsthread_table = NULL;

void nsthread_send_command(NSThread *thread, intercom_Command code, const char
	*data, ...)
{
    va_list ap;

    va_start(ap, data);
    intercom_send_vcommand(thread->kiddy->buf, code, thread->thread_idx,
	    data, ap);
    va_end(ap);
}

NSThread *nsthread_new(ServerData *serv, BOOL upload_first)
{
    NSThread *thrd;
    
    thrd = XMALLOC(sizeof(NSThread));
    thrd->kiddy = kiddy_get_available();
    if (!thrd->kiddy)
    {
	free(thrd);
	return NULL;
    }
    if (!nsthread_table)
    {
	int i;

	nsthread_table = XMALLOC(maincf_get_max_threads() * sizeof(NSThread *));
	for (i = 0; i < maincf_get_max_threads(); ++i)
	    nsthread_table[i] = NULL;
    }
    thrd->kiddy->thread = thrd;
    thrd->serv = serv;
    thrd->thread_idx = thrd->kiddy->thread_idx;
    thrd->upload_first = upload_first;
    if (nscurses_active())
    {
	nscurses_server(thrd->thread_idx, serv->identifier);
    }
    deblog_info_thread(thrd->thread_idx, _("Servicing %s, %s first\n"),
	    serv->identifier, upload_first ? _("uploading") : _("downloading"));
    nsthread_table[thrd->thread_idx] = thrd;
    if (serv->status == server_Unresolved)
    {
	deblog_message_thread(thrd->thread_idx, _("Looking up address of %s\n"),
		serv->address);
	serv->status = server_Resolving;
	nsthread_send_command(thrd, command_Resolve, N_("%s"), serv->address);
    }
    else
    {
	deblog_message_thread(thrd->thread_idx, _("Connecting to %s\n"),
		serv->identifier);
	nsthread_send_command(thrd, command_Connect, N_("%d %s %s"), 
		serv->port, serv->dotquad,
		thrd->serv->path ? thrd->serv->path : N_("!"),
		thrd->serv->pipeline);
	serv->status = server_Connected;
    }
    return thrd;
}

NSThread *nsthread_lookup(int thread_idx)
{
    return nsthread_table[thread_idx];
}

void nsthread_delete(NSThread *thread)
{
    if (!thread)
	return;
    kiddy_detach_thread(thread->kiddy);
    --thread->serv->nthreads;
    free(thread);
}

