/* Parses the config file */

/* $Id: parserc.h,v 2.0 2001/07/26 16:35:12 tony Exp $ */

#ifndef PARSERC_H
#define PARSERC_H

#ifndef DEBLOG_H
#include "deblog.h"
#endif
#ifndef DEFNS_H
#include "defns.h"
#endif

/* When the parser encounters a line beginning with a keyword it calls the
 * handler registered for that keyword with the whole line minus leading
 * whitespace and the data handle associated with that keyword. A handler
 * should return NULL if it reads the line OK, or an error message otherwise.
 * When reported, the error message will have " in line XX: `THELINE'\n"
 * appended
 */
typedef const char *(*parserc_handler)(const char *keyword, const char *value,
	void *data_handle);

void parserc_register_handler(const char *keyword, parserc_handler, void
	*data_handle);

/* Register internal handlers for the various types of line in a newsqrc file
 */

void parserc_register_int(const char *keyword, long int *value);

void parserc_register_bool(const char *keyword, BOOL *value);

/* *value is freeable */
void parserc_register_string(const char *keyword, char **value);

/* This can be used instead of above handlers, if you want to look at every
 * processed line without knowing keywords in advance */
void parserc_register_master_handler(parserc_handler, void *data_handle);

/* Parse the file then deregister all the above handlers */
void parserc_read(int filedes);

/* As above but opens and closes the file; deblog level is for message if file
 * isn't present */
void parserc_read_named(const char *filename, deblog_level_t);

#endif /* PARSERC_H */

