/* Data associated with a server */

/* $Id: server.h,v 2.0 2001/07/26 16:35:12 tony Exp $ */

#ifndef SERVER_H
#define SERVER_H

#ifndef DEFNS_H
#include "defns.h"
#endif

#include <netinet/in.h>

typedef enum {
    server_Unresolved,
    server_Resolving,
    server_Resolved,
    server_Connected,		/* Group info might not be valid yet */
    server_AllDone,
    server_Error		/* An error has occurred, don't use this server
				   again */
} server_Status;

struct ServerData {
    ServerData *next;
    int sidx;			/* Server index number, 0-9, used for
				   identification in interprocess comms */
    server_Status status;
    char *identifier;
    char *address;
    char *dotquad;
    long int port;
    char *user;
    char *password;
    char *path;
    BOOL mode_reader;
    BOOL enter_group;
    BOOL pipeline;
    long int max_threads;
    long int server_weight;
    long int nthreads;		/* Number of threads running */
    long int group_ix;		/* Which group is having articles downloaded */
    long int next_group_stats;	/* Next group to be returned by
				   server_get_next_group_stats */
    long int ngroups;		/* Number of groups in use */
    long int group_size;	/* Total number of elements available in group,
				   not necessarily all occupied */
    GroupData **group;		/* Vector of newsgroups */
    BOOL feed;			/* Whether to check for outgoing articles */
    long int noutgoing;		/* Number of outgoing articles */
    long int outgoing_size;	/* Number of elements in outgoing vector */
    long int nuploaded;		/* Number of outgoings uploaded */
    char **outgoing;		/* Vector of outgoing article leaf names */
};

ServerData *server_new(const char *identifier);

void server_delete(ServerData *);

void server_add_group(ServerData *, GroupData *);

/* Adds all groups from a server's newsrc file */
void server_read_newsrc(ServerData *);

/* Converse of above, writes newsrc file after fetch */
void server_write_newsrc(ServerData *);

/* Reads options from a server-specific cf file */
void server_read_cf(ServerData *);

/* Scans RC_DIR to find all newsrc files */
void server_scan_rc_dir(void);

/* Returns most eligible server due to be serviced by a thread */
typedef enum {
    server_GFT_OK,		/* Connect to this server */
    server_GFT_AllBusy,		/* All currently busy, or with insufficient
				   info to be connected to, but not yet
				   finished */
    server_GFT_AllDone		/* All done, can exit now */
} server_GFT_Result;
server_GFT_Result server_get_for_threading(ServerData **);

/* Returns index of server's next group to get stats and Message-IDs for. If no
 * groups are available, returns -1. Any returned group will have status
 * updated to Allocated. */
long int server_get_next_group_stats(ServerData *);

/* Returns index of server's next group with known stats to fetch articles
 * from. If no groups are available (all AllDone), returns -1. */
long int server_get_next_group_articles(ServerData *);

#endif /* ! SERVER_H */
