/* Enhancements and extensions to std C library */

/* $Id: stdx.h,v 2.2 2001/08/31 16:38:54 tony Exp $ */

#ifndef STDX_H
#define STDX_H

#include <stdio.h>

#ifndef DEFNS_H
#include "defns.h"
#endif

#include <sys/time.h>

/* These log error message and exit if fail. */
void *xmalloc(size_t, const char *msg, ...);
void *xrealloc(void *, size_t, const char *msg, ...);
#define XMALLOC(s) xmalloc((s), _("Out of memory\n"))
#define XREALLOC(b, s) xrealloc((b), (s), _("Out of memory\n"))

char *xstrdup(const char *s, const char *msg, ...);
#define XSTRDUP(s) xstrdup((s), _("Out of memory\n"))

char *xstrdupcat(const char *s1, const char *s2, const char *msg, ...);
#define XSTRDUPCAT(s1, s2) xstrdupcat((s1), (s2), _("Out of memory\n"))

int xpipe(int fd[2], const char *msg, ...);
#define XPIPE(fd2) xpipe((fd2), _("Unable to open pipes"));

/* Finds the difference between 2 times */
void xtimeval_subtract(struct timeval *higher, struct timeval *lower, struct
	timeval *result); 
/* Converts a timeval to floating point, ignoring anything over an hour */
#define FLOAT_TV(ftv) \
    ((double) ((ftv)->tv_sec % 3600) + (double) (ftv)->tv_usec / 1E6)

/* Converts a number into K/M etc notation if it's big; result is overwritten
 * by susequent calls; recursive, suffix must be 0 on entry */
const char *xntokilomeg(long int n, char suffix);
#define XNTOKILOMEG(n) xntokilomeg((n), 0)

char *xtempnam(const char *dir, const char *pfx, const char *msg, ...);
#define XTEMPNAM(d, p) xtempnam((d), (p), _("Can't create temporary\n"))

#define XMAX(a, b) ((a) >= (b) ? (a) : (b))
#define XMIN(a, b) ((a) <= (b) ? (a) : (b))

/* Rest aren't used yet and need fixing */

#if 0
FILE *xfopen(const char *path, const char *mode, const char *msg, ...);
#define XFOPEN(p, m) xfopen((p), (m), _("Unable to open file `%s'"), (p))
char *xfgets(char *, int, FILE *, const char *msg, ...);
#define XFGETS(c, l, fp, fn) xfgets((c), (l), (fp), M_READFILE, (fn))

void xchdir(const char *dir, const char *msg, ...);
#define XCHDIR(d) xchdir((d), M_CHDIR, (d))

/* Unlike previous calls, following don't exit on error. */

/* Returns 0 for failure, != 0 for success. */
int xfcopy(const char *src, const char *dest, const char *msg, ...);
#define XFCOPY(src, dest) xfcopy((src), (dest), M_FCOPY, (src), (dest))

/* Return value as for std rename */
int xrename(const char *src, const char *dest, const char *msg, ...);
#define XRENAME(src, dest) xrename((src), (dest), M_RENAME, (src), (dest))

int xremove(const char *filename, const char *msg, ...);
#define XREMOVE(f) xremove((f), M_REMOVE)

#endif /* Comment block */

#endif /* STDX_H */
